﻿using System.Linq.Expressions;
using System.Windows;
using System.Windows.Controls;
using DynamicExpressionEditorExample.Expressions;
using Telerik.Data;
using Telerik.Data.Expressions;
using Expression = System.Linq.Expressions.Expression;

namespace DynamicExpressionEditorExample
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			this.InitializeComponent();

			var dataTable = new DataTable();
			dataTable.Columns.Add(new DataColumn { ColumnName = "A", DataType = typeof(int) });
			dataTable.Columns.Add(new DataColumn { ColumnName = "B", DataType = typeof(int) });
			dataTable.Columns.Add(new DataColumn { ColumnName = "BA", DataType = typeof(int) });
			dataTable.Columns.Add(new DataColumn { ColumnName = "Max", DataType = typeof(int) });

			var row = dataTable.NewRow();
			row["A"] = 2;
			row["B"] = 3;
			row["BA"] = 5;
			row["Max"] = 8;

			// Another way to set them using dynamic
			//dynamic rowData = row.RowObject;
			//rowData.A = 2;
			//rowData.B = 3;
			//rowData.BA = 5;

			dataTable.Rows.Add(row);

			this.DataContext = dataTable.ToList()[0];
		}

		private void OnTransformClicked(object sender, RoutedEventArgs e)
		{
			if (this.ExpressionEditor.IsExpressionValid)
			{
				var expression = (LambdaExpression)this.ExpressionEditor.Expression;
				var bindedExpression = BindParameter(expression, this.DataContext);

				var evaluatedExpression = (LambdaExpression) Evaluator.PartialEval(bindedExpression, this.CanEvalPartially);

				MessageBox.Show(evaluatedExpression.Body.ToString());
			}
		}
  
		private static Expression BindParameter(LambdaExpression expression, object parameterValue)
		{
			return new ParameterBinderVisitor(expression.Parameters[0], parameterValue).Visit(expression);
		}

		private bool CanEvalPartially(Expression expression)
		{
			if (expression.NodeType == ExpressionType.MemberAccess)
			{
				var member = (MemberExpression)expression;
				if (member.Expression != null && member.Expression.NodeType == ExpressionType.Constant)
				{
					return ((ConstantExpression)member.Expression).Value == this.DataContext;
				}
			}

			return expression.NodeType == ExpressionType.Constant;
		}
	}
}